package com.yiqi.system.service.impl;

import java.util.List;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.yiqi.common.utils.SecurityUtils;
import com.yiqi.system.domain.SysConfigRecord;
import com.yiqi.system.domain.SysConfigTab;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.yiqi.system.mapper.SysDataGroupMapper;
import com.yiqi.system.domain.SysDataGroup;
import com.yiqi.system.service.ISysDataGroupService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

/**
 * 组合数据Service业务层处理
 *
 * @author douya
 * @date 2024-10-29
 */
@Service
public class SysDataGroupServiceImpl extends ServiceImpl<SysDataGroupMapper, SysDataGroup> implements ISysDataGroupService {

    //<editor-folder desc="基础韩素">

    /**
     * 查询组合数据
     *
     * @param id 组合数据主键
     * @return 组合数据
     */
    @Override
    public SysDataGroup selectSysDataGroupById(Integer id) {
        return baseMapper.selectSysDataGroupById(id);
    }

    /**
     * 查询组合数据列表
     *
     * @param sysDataGroup 组合数据
     * @return 组合数据
     */
    @Override
    public List<SysDataGroup> selectSysDataGroupList(SysDataGroup sysDataGroup) {
        return baseMapper.selectSysDataGroupList(sysDataGroup);
    }

    /**
     * 新增组合数据
     *
     * @param sysDataGroup 组合数据
     * @return 结果
     */
    @Override
    public int insertSysDataGroup(SysDataGroup sysDataGroup) {
        return baseMapper.insertSysDataGroup(sysDataGroup);
    }

    /**
     * 修改组合数据
     *
     * @param sysDataGroup 组合数据
     * @return 结果
     */
    @Override
    public int updateSysDataGroup(SysDataGroup sysDataGroup) {
        return baseMapper.updateSysDataGroup(sysDataGroup);
    }

    /**
     * 批量删除组合数据
     *
     * @param ids 需要删除的组合数据主键
     * @return 结果
     */
    @Override
    public int deleteSysDataGroupByIds(Integer[] ids) {
        return baseMapper.deleteSysDataGroupByIds(ids);
    }

    /**
     * 删除组合数据信息
     *
     * @param id 组合数据主键
     * @return 结果
     */
    @Override
    public int deleteSysDataGroupById(Integer id) {
        return baseMapper.deleteSysDataGroupById(id);
    }


    @Override
    public SysDataGroup selectSysDataGroupByKey(String groupKey) {
        QueryWrapper<SysDataGroup> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("config_name", groupKey);
        return baseMapper.selectOne(queryWrapper);
    }
}
